@@ -7,6 +7,7 @@ import boto3
 import botocore
 import logging
 import subprocess
+import traceback
 import json
 import yaml
 import shutil
@@ -19,7 +20,7 @@ logging.basicConfig(level=logging.INFO)
 
 # exception hook so errors raise to main thread
 def exception_hook(args):
-    raise args.exc_type(args.exc_value)
+    traceback.print_exc()
     sys.exit(1)
 
 # set the exception hook
@@ -34,7 +35,7 @@ class EC2Manager:
         self._github_token = os.environ.get('GITHUB_TOKEN', kwargs.get('github_token'))
 
         # optional envs
-        self._config_file = os.environ.get('EC2_MANAGER_CONFIG', kwargs.get('config', 'config.yaml'))
+        self._config_file = os.environ.get('EC2_MANAGER_CONFIG', kwargs.get('config') or 'config.yaml')
         self._commit = os.environ.get('COMMIT', 'master')
         self._terraform_directory = os.environ.get(
             'TERRAFORM_DIRECTORY', os.path.join(os.getcwd(), 'terraform')
@@ -54,16 +55,16 @@ class EC2Manager:
         if self._config_file:
             # internal variables
             self._config = self._get_config()
-            self._aws_default_region = self._config['aws_region']
-            self._type = self._config['type']
-            self._vpc_name = self._config['vpc_name']
-            self._public_subnet_cidr = self._config['public_subnet_cidr']
-            self._expose_docker_daemons = int(self._config.get('expose_docker_daemons'))
+            self._aws_default_region = self._config.get('aws_region')
+            self._type = self._config.get('type')
+            self._vpc_name = self._config.get('vpc_name')
+            self._public_subnet_cidr = self._config.get('public_subnet_cidr')
+            self._expose_docker_daemons = int(self._config.get('expose_docker_daemons', 0))
             self._private_subnet_cidr = self._config.get('private_subnet_cidr', '')
             self._instance_names = self._get_instance_names()
             self._instance_repo_updates = self._get_instance_repo_updates()
 
-            # boost the pool size of urllib to handle concurrent for each instance
+            # boost the pool size of urllib to handle as many concurrent connection as there are instances
             client_config = botocore.config.Config(
                 max_pool_connections=len(self._instance_names),
             )
@@ -72,6 +73,11 @@ class EC2Manager:
             self._ec2_client = self._aws_session.client('ec2', config=client_config)
             self._s3_client = self._aws_session.client('s3', config=client_config)
 
+            self._github_client = Github(
+                login_or_token=self._github_token,
+                pool_size=len(self._instance_names)
+            )
+
             account_number = self._aws_session.client('sts').get_caller_identity().get('Account')
             if not account_number:
                 raise ValueError(
@@ -88,26 +94,10 @@ class EC2Manager:
         :returns: A dictionary of file data.
         :rtype: dict
         """
-        with open(self._config_file, 'r') as config_file:
-            return yaml.safe_load(config_file)
-
-    def _get_json_from_s3(self, bucket_name, file_name):
-        """
-        Gets a json file from an s3 bucket.
-
-        :param str bucket_name: The s3 bucket name.
-        :param str file_name: The s3 file object name.
-        :returns: A dictionary of file data.
-        :rtype: dict
-        """
-        s3 = boto3.resource('s3')
-        content_object = s3.Object(bucket_name, file_name)
-        try:
-            file_content = content_object.get()['Body'].read().decode('utf-8')
-            return json.loads(file_content)
-        except Exception:
-            self._logger.warning('backend state file does not exist.')
-            return {}
+        if os.path.exists(self._config_file):
+            with open(self._config_file, 'r') as config_file:
+                return yaml.safe_load(config_file)
+        return {}
 
     @staticmethod
     def _get_encoded_envs(encoded_string):
@@ -158,14 +148,20 @@ class EC2Manager:
             instance_id = self._get_instance_id(name, try_once, attempts + 5)
         return instance_id
 
-    def _get_terraform_outputs(self):
+    def get_terraform_outputs(self):
         """
-        Gets the outputs from terraform state.
+        Gets the output from terraform state.
 
-        :return dict: A dictionary of output values.
+        :returns: Terraform outputs.
+        :rtype: dict
         """
-        state_data = self._get_json_from_s3(self._aws_backend_bucket, self._aws_backend_key)
-        return state_data.get('outputs', {})
+        self._logger.info(f'Exporting Terraform outputs...')
+        output = subprocess.check_output(
+            ['terraform', 'output', '-json'],
+            env=os.environ,
+            cwd=self._terraform_directory
+        )
+        return json.loads(output)
 
     def _get_instance_names(self):
         """
@@ -218,8 +214,6 @@ class EC2Manager:
                 'NAME': name,
                 'REPO': self._repo,
                 'COMMIT': self._commit,
-                'GITHUB_USERNAME': self._github_user,
-                'GITHUB_TOKEN': self._github_token,
                 **instances[name].get('envs', {}),
                 **self._get_encoded_envs(os.environ.get(instances[name].get('encoded_env_file_variable', ''), ''))
             }
@@ -268,8 +262,7 @@ class EC2Manager:
             'terraform',
             'init',
             f'-backend-config=key={self._aws_backend_key}',
-            f'-backend-config=bucket={self._aws_backend_bucket}',
-            f'-backend-config=region={self._aws_default_region}'
+            f'-backend-config=bucket={self._aws_backend_bucket}'
         ],
             env=os.environ,
             cwd=self._terraform_directory
@@ -281,8 +274,8 @@ class EC2Manager:
         """
         Runs terraform apply that creates the ec2 instances.
         """
-        outputs = self._get_terraform_outputs()
-        if outputs.get('instance_names', {}).get('value') != self._instance_names:
+        outputs = self.get_terraform_outputs()
+        if outputs.get('instances', {}).get('value') != self._instance_names:
             self._logger.info(f'Terraform instances...')
             instances = self.encode_string(json.dumps(self._get_instance_data()))
 
@@ -317,7 +310,7 @@ class EC2Manager:
         Helper method for calling a given method on all provided.
 
         :param callable callable_instance: A callable.
-        :param list items: A list of items to call.
+        :param Any items: A list of items to call.
         :param list extra_args: A list of extra arguments.
         :param bool join: Whether or not to join the threads.
         :return list[Thread]: A list of threads.
@@ -465,14 +458,15 @@ class EC2Manager:
         """
         if self._repo not in self.run_command(name, ['ls'], print_output=False):
             self._logger.info(f'{name} cloning {self._repo}')
+            print(f'git clone {self._repo_url}')
             self.run_command(name, [f'git clone {self._repo_url}'])
 
         self._logger.info(f'{name} checking out {self._commit}')
         self.run_command(name, [
             f'cd ./{self._repo}',
             'git fetch',
-            'git pull origin master',
-            f'git -c advice.detachedHead=false checkout {self._commit}'
+            'git pull',
+            # f'git -c advice.detachedHead=false checkout {self._commit}'
         ])
 
     def _stop(self, name, data):
