@@ -81,9 +81,9 @@ override_file = click.option(
 )
 
 
-def get_class(**kwargs):
+def get_class(kwargs):
     config_file = kwargs.get('config')
-    file_path = kwargs.get('override_file', 'override.py')
+    file_path = kwargs.get('override_file') or 'override.py'
 
     if not os.path.exists(config_file):
         raise click.UsageError(f'The config file "{config_file}" was not found.')
@@ -224,7 +224,7 @@ def copy_env(**kwargs):
 @github_token
 @override_file
 def apply(**kwargs):
-    ec2_manager_instance = get_class(**kwargs)()
+    ec2_manager_instance = get_class(kwargs)(**kwargs)
     ec2_manager_instance.apply()
 
 
